﻿#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3				// Use modern global access method and strict wave access
#pragma DefaultTab={3,20,4}		// Set default tab width in Igor Pro 9 and later
#pragma version		= 1.5
#pragma IndependentModule = QGraphPop
//
//	https://www.wavemetrics.com/comment/21112#comment-21112 
// 
//	remodified by Q 20220405 : 
//	It can be used for standalone, or attach to QgraphSizepanle module,
//	by simultaneously including.
//
//	2023-07-29 -	ver.    :		release
//
//
//
Menu "TracePopup", dynamic
    "Copy Trace", /Q, CopyTrace()
    SelectString(cmpstr(GetScrapText()[0,5], "YWAVE:")==0, "", "Paste Trace Formatting"), /Q, PasteTraceFormatting(2)
    SelectString(cmpstr(GetScrapText()[0,5], "YWAVE:")==0, "", "Paste Trace Offsets"), /Q, PasteTraceFormatting(1)
    SelectString(cmpstr(GetScrapText()[0,5], "YWAVE:")==0, "", "Paste Trace errorbar"), /Q, PasteTraceFormatting(0)

    "Fixed to Dir", /Q, FixedToDir()
    "Offsets Between Corsurs", /Q, offsetsAmongCorsurAB()
end
//
Menu "GraphPopup", dynamic
//    "Print Graph Path ##", /Q, PrintSubWinPath(3)
	"---"
    SelectString(cmpstr(GetScrapText()[0,5], "YWAVE:")==0, "", "Paste Trace"), /Q, PasteTrace()
    SelectString(cmpstr(GetScrapText()[0,5], "YWAVE:")==0, "", "Paste SwapXY Trace"), /Q, PasteSwapXYTrace()
    
   	OffsetsABName(),/Q, DooffsetsAmongCorsurAB()
end
//
Menu "TracePopup", dynamic	// menu when a trace is right-clicked
	"---"	// separator divides this from built-in menu items
	ExportTraceName(),/Q, ExportSelectedTrace()
//	"Draw XY Here", DrawXYHere()
End
//#########--**--        ————————————          --**--#########
////
Function FixedToDir()
// 220914 - Changing the function policy
// 			
   	GetLastUserMenuInfo
    wave w = TraceNameToWaveRef(S_graphName,S_traceName)  

//	in case not a trace
    if(strlen(nameofWave(w)) == 0)  //note that only works for the top image in top window
//    	str2num(StringByKey("margin(bottom)", gRec, "=", ","))
    	String s=ImageNameList("",";")
    	Wave w=ImageNameToWaveRef("",StringFromList(0,s))
//		string s = imageinfo("", "", 5)
//		print s
    endif
//
    string s_folder = GetWavesDataFolder(w,1) //+ ";" + TraceInfo(S_graphName,S_traceName,0)  
//    PutScrapText s_folder
//    print "s = ", s_folder		// debug 
    setDataFolder  s_folder
    print "\r\t\tCurrentDataFolder was Fixed to ", s_folder
    print "\r\t\t\t\ttrace name is ", S_traceName
end
//#########--**--        ————————————          --**--#########
//
Function/S ExportTraceName()
	GetLastUserMenuInfo	// Sets S_graphName, S_traceName, V_mouseX, V_mouseY 
	if (strlen(S_traceName) > 0)
		String item = "Export "+S_traceName
		return item
	endif
	return ""		// No item is added to menu
End
//
Function/S OffsetsABName()
	string wnameA,wnameB
	
	wnameA=CsrWave(A) // name of wave on which user has placed cursor A
	wnameB=CsrWave(B)
	
	if (stringmatch(wnameA, wnameB))
		String item = "Axis Offsets Between Cursors "
		return item
	endif
	return ""		// No item is added to menu
End
//
Function ExportSelectedTrace()
	GetLastUserMenuInfo
	// Do something with S_graphName, S_traceName
    
    CopyTrace()
    
    string s = "", cmd = "Display ", info = GetScrapText()
    cmd += StringByKey("YWAVE", info, ":", ";") + " vs "
    cmd += StringByKey("XWAVEDF", info, ":", ";") + "'" + StringByKey("XWAVE", info, ":", ";") + "'"
	execute cmd
	PasteTraceFormatting(2)
	PasteTraceFormatting(1)
End
//
function CopyTrace()
    GetLastUserMenuInfo
    wave w = TraceNameToWaveRef(S_graphName,S_traceName)   
    string s = "YWAVE:" + GetWavesDataFolder(w,4) + ";" + TraceInfo(S_graphName,S_traceName,0)
    PutScrapText s
//    print "s = ", s
end
//
function PasteTrace()
    string s = "", cmd = "AppendToGraph", info = GetScrapText()
    cmd += StringByKey("AXISFLAGS", info, ":", ";") + " "
    cmd += StringByKey("YWAVE", info, ":", ";") + StringByKey("YRANGE", info, ":", ";")
    s = StringByKey("XWAVE", info, ":", ";")
    if (strlen(s))
    	s = "'" + s + "'"
        cmd += " vs " + StringByKey("XWAVEDF", info, ":", ";") + s + StringByKey("XRANGE", info, ":", ";")
    endif  
    execute cmd
    
    s=StringByKey("ERRORBARS", info, ":", ";")
    if (strlen(s))
        execute s
    endif

    // figure out trace name - hopefully the last 'normal' trace is the one we just added
    s = TraceNameList("",";",1)
    string tracename = StringFromList(itemsinlist(s)-1, s)
   
    // apply formatting
    s = info[strsearch(info, "RECREATION:", 0) + 11, Inf]
    int i
    int numCmd = itemsinlist(s)
    for(i=0;i<numCmd;i++)
        cmd = StringFromList(i, s)
        if (!stringmatch(cmd, "offset(x)=*"))
            Execute /Q/Z "ModifyGraph " + ReplaceString("(x)", cmd, "("+traceName+")", 0, 1)
        endif
    endfor
    
    print "\r//--**-- Paste " + StringByKey("YWAVE", info, ":", ";")+" vs "+StringByKey("XWAVE", info, ":", ";")
end
//
function PasteSwapXYTrace()
    string s = "", cmd = "AppendToGraph /VERT", info = GetScrapText()
    cmd += StringByKey("AXISFLAGS", info, ":", ";") + " "
    cmd += StringByKey("YWAVE", info, ":", ";") + StringByKey("YRANGE", info, ":", ";")
    s = StringByKey("XWAVE", info, ":", ";")
    if (strlen(s))
    	s = "'" + s + "'"	// forced standard refrence
        cmd += " vs " + StringByKey("XWAVEDF", info, ":", ";") + s + StringByKey("XRANGE", info, ":", ";")
    endif  
    execute cmd
    
    s=StringByKey("ERRORBARS", info, ":", ";")
    if (strlen(s))
        execute s
    endif

    // figure out trace name - hopefully the last 'normal' trace is the one we just added
    s = TraceNameList("",";",1)
    string tracename = StringFromList(itemsinlist(s)-1, s)
   
    // apply formatting
    s = info[strsearch(info, "RECREATION:", 0) + 11, Inf]
    int i
    int numCmd = itemsinlist(s)
    for(i=0;i<numCmd;i++)
        cmd = StringFromList(i, s)
        if (!stringmatch(cmd, "offset(x)=*"))
            Execute /Q/Z "ModifyGraph " + ReplaceString("(x)", cmd, "("+traceName+")", 0, 1)
        endif
    endfor
    
    print "\r//--**-- Paste " + StringByKey("XWAVE", info, ":", ";")+" vs "+StringByKey("YWAVE", info, ":", ";")
end
//
function PasteTraceFormatting(int WhichCmds)
    GetLastUserMenuInfo
    string s = GetScrapText(), cmd = ""
    s = s[strsearch(s, "RECREATION:", 0) + 11, Inf]
    print s
    int numCmd = ItemsInList(s)
    int i
    for(i=0;i<numCmd;i++)
        cmd = StringFromList(i, s)
        if ((WhichCmds&1) && cmpstr(cmd[0,9], "offset(x)=")==0) // bit 0, offsets
            Execute /Q/Z "ModifyGraph " + ReplaceString("(x)", cmd, "("+S_traceName+")", 0, 1)
        elseif((WhichCmds&2) && cmpstr(cmd[0,9], "offset(x)="))
            Execute /Q/Z "ModifyGraph " + ReplaceString("(x)", cmd, "("+S_traceName+")", 0, 1)
        endif
    endfor
end
//
function offsetsAmongCorsurAB()
	ExportSelectedTrace()
	ShowInfo/CP=0
end
//
function DooffsetsAmongCorsurAB()  //  not complete the work
	string tagname,wname,wnamex, taglabel, tracename, info, DFname, CTRname, labelfmt
	variable xwexists, tagvalue, tagyvalue, ii, jj, namehash, pntvalue
	
	wname=CsrWave(A) // name of wave on which user has placed cursor A
	if (StrLen(wname)==0)
		print "Cursor A is not on a wave"
		abort
	else
		wname=GetWavesDataFolder(CsrWaveRef(A),2)
		tracename=NameofWave(CsrWaveRef(A))
	endif
	
	wave Dwave=$wname
end
//#########--**--        ————————————          --**--#########

 